package ast;



public class Add extends Bin {

    /** Construct an addition expression.*/
    public Add(Exp lhs, Exp rhs) {
        super(lhs, rhs);
    }

    /** The string that represents the operation in concrete syntax.*/
    public String operator() {
        return "+";
    }

    @Override
    /**
     * Eval the current node and return a value
     */
    public Value eval() {
        Value l = this.lhs_.eval();
        Value r = this.rhs_.eval();
        if (l instanceof ast.Int) {
            return new Int(l.toInt() + r.toInt());
        } else {
            return new Str(l.toString().substring(0, l.toString().length() - 1) + r.toString().substring(1, r.toString().length()));
        }
    }

        /**
     * Return the type of the current node
     * return ""  if there is a problem with the operation's operands 
     * return "String" or "Integer" type :it depends on the 1 operand
     */
    public String getType() {
        String l = this.lhs_.getType();
        String r = this.rhs_.getType();
        if (((l.equals(Value.stringType)) && !((r.equals(l)))) ||
                ((r.equals(Value.stringType)) && !((l.equals(r))))) {
            System.err.println("invalid use of \""+this.operator()+"\"");
            return "";
        }

        return l;
    }
    
// FIXME: Some code was deleted here (Evaluation, type-checking etc).
}
