/*
 * Class Lss (Less <)
 * Compare two integers i1 < i2
 * Compare two strings  s1 < s2 (ascii order)
 * Fails if two operands have not the same type
 * ex: 2<(1*2) => integer 0
 * ex: 2<3 => integer 1
 * ex: "abc"<"ABC" => integer 0
 */
package ast;

/**
 *
 * @author karim
 */
public class Lss extends ast.Bin {

    /**Constructor*/
    public Lss(Exp lhs, Exp rhs) {
        super(lhs, rhs);
    }

    /** The string that represents the operation in concrete syntax.*/
    @Override
    public String operator() {
        return "<";
    }

    /**
     * Eval the current node and return a value
     * (here , ast.Int 0 or ast.Int 1)
     */
    @Override
    public Value eval() {
        Value l = this.lhs_.eval();
        Value r = this.rhs_.eval();
        if (l instanceof ast.Int) {
            return this.lhs_.eval().toInt() < this.rhs_.eval().toInt() ? new Int(1) : new Int(0);
        } else {
            return this.lhs_.eval().toString().compareTo(this.rhs_.eval().toString()) < 0 ? new Int(1) : new Int(0);
        }
    }

       /**
     * Return the type of the current node
     * return ""  if there is a problem with the operation's operands 
     * else return  "Integer" type 
     */
    @Override
    public String getType() {
        String l = this.lhs_.getType();
        String r = this.rhs_.getType();
        if (((l.equals(Value.stringType)) && !((r.equals(l)))) ||
                ((r.equals(Value.stringType)) && !((l.equals(r))))) {
            System.err.println("invalid use of \"" + this.operator() + "\"");
            return "";
        }
        
        return Value.integerType;
    }
    
    
}
