/*
 * Class Eql (Equal)
 * Test if two integer are equals
 * Test if two strings are equals 
 * Fails if two operands have not the same type
 * ex: 2=(1*2) => integer 1
 * ex: 2=3 => integer 0
 * ex: "abc"="ABC" => integer 0
 */
package ast;

/**
 *
 * @author karim
 */
public class Eql extends ast.Bin {

    /** Constructor need two exp expressions*/
    public Eql(Exp lhs, Exp rhs) {
        super(lhs, rhs);
    }

    /** The string that represents the operation in concrete syntax.*/
    @Override
    public String operator() {
        return "=";
    }

    /**
     * Eval the current node and return a value
     */
    @Override
    public Value eval() {
        Value l = this.lhs_.eval();
        Value r = this.rhs_.eval();
        if (l instanceof ast.Int) {
            return this.lhs_.eval().toInt().equals(this.rhs_.eval().toInt()) ? new Int(1) : new Int(0);
        } else {
            return this.lhs_.eval().toString().compareTo(this.rhs_.eval().toString()) == 0 ? new Int(1) : new Int(0);
        }
    }

    /**
     * Return the type of the current node
     * return ""  if there is a problem with the operation's operands 
     * else return  "Integer" type 
     */
    @Override
    public String getType() {
        String l = this.lhs_.getType();
        String r = this.rhs_.getType();
        if (((l.equals(Value.stringType)) && !((r.equals(l)))) ||
                ((r.equals(Value.stringType)) && !((l.equals(r))))) {
            System.err.println("invalid use of \"" + this.operator() + "\"");
            return "";
        }
        
        return Value.integerType;
    }
}
